package com.starbuds.server.service.feed.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableList;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.feed.FeedActionEnum;
import com.starbuds.server.common.constant.feed.FeedActionTargetEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.RecordList;
import com.starbuds.server.common.pojo.common.KeyValueMapDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.feed.*;
import com.starbuds.server.common.pojo.mq.QueueFeedCreateMsg;
import com.starbuds.server.common.pojo.mq.QueueFeedLikeMsg;
import com.starbuds.server.common.pojo.mq.QueueFeedViewMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.feed.FeedService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.common.tag.feed.UserTag;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.feed.dao.FeedActionDAO;
import com.starbuds.server.service.feed.dao.FeedCommentDAO;
import com.starbuds.server.service.feed.dao.FeedDAO;
import com.starbuds.server.service.feed.dao.FeedThumbsupLogDAO;
import com.starbuds.server.service.feed.model.FeedIdListBO;
import com.starbuds.server.service.feed.provider.FeedGiftRewardProvider;
import com.starbuds.server.service.feed.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 内容服务
 *
 * @author aoe
 * @date 2020/5/27
 */
@Service(version = "1.0.0")
@Slf4j
public class FeedServiceImpl implements FeedService {

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private FeedDAO feedDAO;

    @Resource
    FeedCommentDAO feedCommentDAO;

    @Resource
    FeedActionDAO feedActionDAO;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    FeedThumbsupLogDAO feedThumbsupLogDAO;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Resource
    FeedGiftRewardProvider feedGiftRewardProvider;

    private static FeedIdListBO feedIdListBO = new FeedIdListBO();

    private static Random random = new Random();

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long  createFeed(FeedDO feed, Long channelId) {
        Long feedId = identityService.getId();
        feed.setFeedId(feedId);
        feed.setCreateTime(System.currentTimeMillis());
        feed.setUpdateTime(feed.getCreateTime());
        feed.setIsDeleted(BoolIntVal.False);

        int result = feedDAO.insert(feed);
        if (result != 1) {
            log.error("内容服务创建失败 {}", feed);
            return null;
        }
        log.info("内容服务创建成功 {}", feed);

        // 内容创建
        QueueFeedCreateMsg createMsg = new QueueFeedCreateMsg();
        createMsg.setFeedId(feedId);
        createMsg.setUrl(feed.getUrl());
        createMsg.setFeedType(feed.getFeedType());
        createMsg.setChannelId(channelId);
        createMsg.setUserId(feed.getUserId());
        boolean sendResult = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FEED_CREATE, createMsg.serialize());

        log.info("发送创建内容通知 sendResult={} msg={}", sendResult, createMsg);

        return feedId;
    }

    @Override
    public boolean isFeedExist(Long feedId) {
        return feedDAO.countById(feedId) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addComment(FeedCommentDO commentDO) {
        return feedCommentDAO.insert(commentDO) > 0;
    }

    @Override
    public boolean isCommentExist(Long commentId) {
        return feedCommentDAO.countById(commentId) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addAction(FeedActionDO actionDO) {
        return feedActionDAO.insertOrUpdate(actionDO) >= 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeLikeCount(Long feedId, int count) {
        try {
            Long version = feedDAO.selectVersion(feedId);
            if (version != null) {
                int result = 0;
                if (count > 0) {
                    result = feedDAO.increaseLikeCount(feedId, count, version);
                } else {
                    result = feedDAO.decreaseLikeCount(feedId, count, version);
                }
                if (result == 0) {
                    changeLikeCount(feedId, count);
                }
                return true;
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);

        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeCollectCount(Long feedId, int count) {
        try {
            Long version = feedDAO.selectVersion(feedId);
            int result = 0;
            if (count > 0) {
                result = feedDAO.increaseCollectCount(feedId, count, version);
            } else {
                result = feedDAO.decreaseCollectCount(feedId, count, version);
            }
            if (result == 0) {
                changeCollectCount(feedId, count);
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);

        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeShareCount(Long feedId, int count) {
        try {
            Long version = feedDAO.selectVersion(feedId);
            int result = 0;
            if (count > 0) {
                result = feedDAO.increaseShareCount(feedId, count, version);
            } else {
                result = feedDAO.decreaseShareCount(feedId, count, version);
            }
            if (result == 0) {
                changeShareCount(feedId, count);
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);

        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeCommentCount(Long feedId, int count) {
        try {
            Long version = feedDAO.selectVersion(feedId);
            int result = 0;
            if (count > 0) {
                result = feedDAO.increaseCommentCount(feedId, count, version);
            } else {
                result = feedDAO.decreaseCommentCount(feedId, count, version);
            }
            if (result == 0) {
                changeCommentCount(feedId, count);
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);
        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeCommentLikeCount(Long targetId, int count) {
        try {
            Long version = feedCommentDAO.selectVersion(targetId);
            int result = 0;
            if (count > 0) {
                result = feedCommentDAO.increaseLikeCount(targetId, count, version);
            } else {
                result = feedCommentDAO.decreaseLikeCount(targetId, count, version);
            }
            if (result == 0) {
                changeCommentLikeCount(targetId, count);
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);
        }
        return false;
    }

    @Override
    public boolean isActionExist(Long userId, Long targetId, Integer targetType, Integer actionType) {
        return feedActionDAO.countById(userId, targetId, targetType, actionType) > 0;
    }

    @Override
    public RecordList<FeedVO, Long> getFeedList(Long lastId, int pageSize, Integer feedType, Long userId, Integer isSelf) {

        RecordList<FeedVO, Long> recordList = new RecordList<>(pageSize);

        List<FeedVO> list = feedDAO.selectFeedList(lastId, pageSize, feedType, userId, isSelf);

        List<FeedVO> feedList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            Set<Long> targetIds = list.stream().map(FeedVO::getUserId).collect(Collectors.toSet());
            Map<Long, Boolean> followersMap = socialRelationService.isFollowers(userId, targetIds);

            List<FeedVO> finalFeedList = new ArrayList<>();
            list.stream()
                    .filter(Objects::nonNull)
                    .filter(feed -> StringUtils.isNotBlank(feed.getUrl()))
                    .forEach(vo -> {
                        Arrays.stream(vo.getUrl().split(","))
                                .forEach(url -> {
                                    Long targetId = vo.getUserId();
                                    Long feedId = vo.getFeedId();
                                    vo.setUrl(url);
                                    if (!CollectionUtils.isEmpty(followersMap)) {
                                        vo.setIsFollow(followersMap.get(targetId) ? BoolIntVal.True : BoolIntVal.False);
                                    }
                                    vo.setIsCollect(isCollect(userId, feedId, feedType) ? BoolIntVal.True : BoolIntVal.False);
                                    vo.setIsLike(isLike(userId, feedId, feedType) ? BoolIntVal.True : BoolIntVal.False);

                                    LiveStatusEnum liveStatus = anchorService.getLiveStatus(targetId);
                                    vo.setLiveStatus(liveStatus == null ? null : liveStatus.getValue());

                                    FeedVO resultVO = new FeedVO();
                                    BeanUtils.copyProperties(vo, resultVO);
                                    finalFeedList.add(resultVO);
                                });
                    });
            feedList = setUserInfo(finalFeedList);
        }
        recordList.setList(feedList);
        recordList.setLastId(feedList.size() == 0 ? null : feedList.get(feedList.size() - 1).getFeedId());
        recordList.setPageSize(pageSize);
        return recordList;
    }

    @Override
    public FeedVO getFeedDetail(Long feedId, Integer feedStatus, Integer isDeleted) {
        FeedDO feedDO = feedDAO.selectDetail(feedId, feedStatus, isDeleted);
        if (feedDO == null) {
            return null;
        }

        FeedVO feedVO = new FeedVO();
        BeanUtils.copyProperties(feedDO, feedVO);
        List<FeedVO> list = new ArrayList<>(1);
        list.add(feedVO);
        list = setUserInfo(list);
        return list.get(0);
    }

    @Override
    public FeedSquareVO getFeedDetail(Long feedId, Long mySelfUserId) {
        List<Long> feedIds = new ArrayList<>();
        feedIds.add(feedId);
        List<FeedSquareVO> res = getFeedSquareVOByFeedIds(feedIds, mySelfUserId);
        if (CollectionUtil.isNotEmpty(res)) {
            return res.get(0);
        } else {
            return null;
        }
    }


    @Override
    public boolean isCollect(Long userId, Long targetId, Integer targetType) {
        return feedActionDAO.countByIdAndStatus(userId, targetId, targetType, FeedActionEnum.COLLECT.getCode(), CommonStatus.Normal) > 0;
    }

    @Override
    public boolean isLike(Long userId, Long targetId, Integer targetType) {
        return feedActionDAO.countByIdAndStatus(userId, targetId, targetType, FeedActionEnum.LIKE.getCode(), CommonStatus.Normal) > 0;
    }

    @Override
    public RecordList<FeedCommentVO, Long> getCommentList(Long feedId, Long curUserId, Long lastId, int pageSize) {
        RecordList<FeedCommentVO, Long> recordList = new RecordList<>(pageSize);
        List<FeedCommentVO> list = feedCommentDAO.selectCommentList(feedId, lastId, pageSize);
        if (!CollectionUtils.isEmpty(list)) {
            // 作者
            Long userId = feedDAO.selectUserId(feedId);
            // replyId => commentList
            Map<Long, List<FeedCommentVO>> map = new HashMap<>();
            for (FeedCommentVO vo : list) {
                if (CollectionUtils.isEmpty(vo.getCommentList())) {
                    Integer commentCount = vo.getCommentCount();
                    Long commentId = vo.getCommentId();
                    if (commentId != null && commentId > 0 && commentCount.equals(1)) {
                        // 如果子评论只有1条的话，只显示一条子评论
                        List<FeedCommentVO> replyList = feedCommentDAO.selectOneByReplyId(commentId);
                        if (!CollectionUtils.isEmpty(replyList)) {
                            for (FeedCommentVO reply : replyList) {
                                reply.setIsAuthor(reply.getUserId().equals(userId) ? BoolIntVal.True : BoolIntVal.False);
                                Long replyUserId = reply.getReplyUserId();
                                if (replyUserId != null && replyUserId > 0) {
                                    UserBasicInfo basicInfo = userService.getBasicInfo(replyUserId);
                                    if (basicInfo != null) {
                                        reply.setReplyUserName(basicInfo.getUserName());
                                        reply.setReplayUserAvatar(basicInfo.getUserAvatar());
                                        reply.setIsLike(isLike(userId, vo.getCommentId(), FeedActionTargetEnum.COMMENT.getValue()) ? BoolIntVal.True : BoolIntVal.False);
                                    }
                                }
                            }
                            replyList = setUserInfo(replyList);
                            map.put(commentId, replyList);
                        }
                    }
                }
            }

            for (FeedCommentVO vo : list) {
                vo.setCommentList(Optional.ofNullable(map.get(vo.getCommentId())).orElse(new ArrayList<>()));
                vo.setIsLike(isLike(curUserId, vo.getCommentId(), FeedActionTargetEnum.COMMENT.getValue()) ? BoolIntVal.True : BoolIntVal.False);
                vo.setIsAuthor(vo.getUserId().equals(userId) ? BoolIntVal.True : BoolIntVal.False);
                Long replyUserId = vo.getReplyUserId();
                if (replyUserId != null && replyUserId > 0) {
                    UserBasicInfo basicInfo = userService.getBasicInfo(replyUserId);
                    if (basicInfo != null) {
                        vo.setReplyUserName(basicInfo.getUserName());
                        vo.setReplayUserAvatar(basicInfo.getUserAvatar());
                    }
                }
            }
        }
        recordList.setList(setUserInfo(list));
        recordList.setLastId(list.size() == 0 ? null : list.get(list.size() - 1).getCommentId());
        recordList.setPageSize(pageSize);
        return recordList;
    }

    @Override
    public Long getUserId(Long feedId) {
        return feedDAO.selectUserId(feedId);
    }

    @Override
    public boolean delFeed(Long feedId) {
        return feedDAO.updateDeleteStatus(feedId, BoolIntVal.True) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addViewCount(Long feedId, Long channelId) {
        QueueFeedViewMsg viewMsg = new QueueFeedViewMsg();
        viewMsg.setChannelId(channelId);
        viewMsg.setFeedId(feedId);
        viewMsg.setFeedType(feedDAO.selectFeedType(feedId));
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FEED_VIEW, viewMsg.serialize());
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delComment(Long commentId) {
        return feedCommentDAO.updateStatus(commentId, CommonStatus.Deleted) > 0;
    }

    @Override
    public Long getCommentUserId(Long commentId) {
        return feedCommentDAO.selectUserId(commentId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeFeedCommentCount(Long commentId, int count) {
        try {
            Long version = feedCommentDAO.selectVersion(commentId);
            int result = 0;
            if (count > 0) {
                result = feedCommentDAO.increaseCommentCount(commentId, count, version);
            } else {
                result = feedCommentDAO.decreaseCommentCount(commentId, count, version);
            }
            if (result == 0) {
                changeFeedCommentCount(commentId, count);
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("更新出错", e);
        }
        return false;
    }

    @Override
    public Long getFeedId(Long commentId) {
        return feedCommentDAO.selectFeedId(commentId);
    }

    @Override
    public RecordList<FeedCommentVO, Long> getChildCommentList(Long lastId, Long curUserId, Long commentId, int pageSize) {
        RecordList<FeedCommentVO, Long> recordList = new RecordList<>(pageSize);
        List<FeedCommentVO> commentList = feedCommentDAO.selectByReplyIdPageList(lastId, commentId, pageSize);
        if (!CollectionUtils.isEmpty(commentList)) {
            // 内容id
            Long feedId = commentList.get(0).getFeedId();
            // 作者
            Long userId = feedDAO.selectUserId(feedId);
            for (FeedCommentVO vo : commentList) {
                vo.setIsAuthor(vo.getUserId().equals(userId) ? BoolIntVal.True : BoolIntVal.False);
                vo.setIsLike(isLike(curUserId, vo.getCommentId(), FeedActionTargetEnum.COMMENT.getValue()) ? BoolIntVal.True : BoolIntVal.False);
                Long replyUserId = vo.getReplyUserId();
                if (replyUserId != null && replyUserId > 0) {
                    UserBasicInfo basicInfo = userService.getBasicInfo(replyUserId);
                    if (basicInfo != null) {
                        vo.setReplyUserName(basicInfo.getUserName());
                        vo.setReplayUserAvatar(basicInfo.getUserAvatar());
                    }
                }
            }
        }
        recordList.setList(setUserInfo(commentList));
        recordList.setPageSize(pageSize);
        recordList.setLastId(commentList.size() == 0 ? null : commentList.get(commentList.size() - 1).getCommentId());
        return recordList;
    }

    @Override
    public Long getReplyId(Long commentId) {
        return feedCommentDAO.selectReplyId(commentId);
    }

    @Override
    public Long getOriReplyId(Long commentId) {
        return feedCommentDAO.selectOriReplyId(commentId);
    }

    /**
     * 获取内容
     *
     * @param feedId
     * @return
     */
    @Override
    public FeedDO getFeed(Long feedId) {
        return feedDAO.selectById(feedId);
    }

    /**
     * 设置照片墙
     *
     * @param feedId
     * @param photoWall
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setPhotoWall(Long feedId, Integer photoWall) {
        return feedDAO.updatePhotoWall(feedId, photoWall) > 0;
    }

    /**
     * 获取照片墙
     *
     * @param userId
     * @return
     */
    @Override
    public List<String> getPhotoWall(Long userId) {
        List<String> photos = feedDAO.getPhotoWalls(userId);
        if (CollectionUtils.isEmpty(photos)) {
            return Collections.EMPTY_LIST;
        }
        return photos.stream()
                .filter(Objects::nonNull)
                .map(url -> url.split(","))
                .flatMap(Arrays::stream)
                .collect(Collectors.toList());
    }

    @Override
    public Map<Long, List<String>> getPhotoWallByUserIds(Collection userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, FeedVO> map = feedDAO.selectPhotoWallByUserIds(userIds);
        if (CollectionUtils.isEmpty(map)) {
            return Collections.EMPTY_MAP;
        }
        log.info("根据用户id获取照片墙 map {}", JSON.toJSONString(map));
        Map<Long, List<String>> result = new HashMap<>(map.size());
        map.forEach((userId, feed) -> {
            if (feed == null) {
                return;
            }
            String urls = feed.getUrl();
            if (StringUtils.isBlank(urls)) {
                return;
            }
            result.put(userId, Stream.of(urls.split(",")).collect(Collectors.toList()));
        });
        return result;
    }

    /**
     * 获取照片墙数量
     *
     * @param userId
     * @return
     */
    @Override
    public int getPhotoWallCount(Long userId) {
        return feedDAO.countPhotoWall(userId);
    }

    @Override
    public PageList<FeedSquareVO> getFeedByDatePageList(Long userId, Long mySelfUserId, Integer feedStatus, int pageNum, int pageSize) {
        PageList<FeedSquareVO> page = new PageList<>(pageNum, pageSize);
        List<Long> feedIds = feedDAO.selectFeedListGroupByDate(page.getPageCursor(), pageSize, userId, feedStatus);
        page.setList(getFeedSquareVOByFeedIds(feedIds, mySelfUserId));
        page.setTotal(feedDAO.selectFeedListGroupByDateCount(userId, feedStatus));
        return page;
    }

    private List<FeedSquareVO> getFeedSquareVOByFeedIds(List<Long> feedIds, Long userId) {

        if (CollectionUtil.isNotEmpty(feedIds)) {
            List<FeedSquareVO> feedList = feedDAO.getSquareFeedListByIds(feedIds);
            if (CollectionUtil.isNotEmpty(feedList)) {
                Set<Long> userIds = new HashSet<>(feedList.size());
                userIds.add(userId);
                feedList.forEach(i -> userIds.add(i.getUserId()));

                // 送礼数量
                Map<Long, String> feedSendGiftNumsMap = feedGiftRewardProvider.giftRewardMap(feedIds);
                // 是否送礼
                Set<Long> hasFeedSendGiftMap = feedGiftRewardProvider.rewardedMap(userId, feedIds);
                // 点赞数量
                Map<Long, Integer> thumbsUpNum = getThumbsNumMap(feedIds);
                // 用户是否点赞
                Set<Long> hasThumbsUpNum = feedThumbsupLogDAO.getThumbsUpSet(feedIds, userId);
                // 用户技能判断
                Map<Long, Integer> skillMap = familyService.getUserSkillMap(userIds);
                // 是否是粉丝
                Set<Long> followSet = socialRelationService.getFollowSet(userId, userIds);

                Map<Long, UserDO> userMap = userService.getUserMapByIds(userIds);

                Set<Long> adminIdSet = userService.getAdminSet(userIds);

                feedList.forEach(i -> {
                    if (followSet.contains(i.getUserId())) {
                        i.setIsFans(BoolIntVal.True);
                    } else {
                        i.setIsFans(BoolIntVal.False);
                    }
                    if (hasThumbsUpNum.contains(i.getFeedId())) {
                        i.setHasThumbsUp(BoolIntVal.True);
                    } else {
                        i.setHasThumbsUp(BoolIntVal.False);
                    }
                    if (hasFeedSendGiftMap.contains(i.getFeedId())) {
                        i.setHasSendGift(BoolIntVal.True);
                    } else {
                        i.setHasSendGift(BoolIntVal.False);
                    }
                    i.setSendGiftNum(handlerStr(feedSendGiftNumsMap.get(i.getFeedId())));
                    i.setThumbsUpNum(handlerThumbsUpNum(thumbsUpNum.get(i.getFeedId())));
                    i.setHasSkills(handleInteger(skillMap.get(i.getUserId())));
                    i.setMySelfHasSkills(handleInteger(skillMap.get(userId)));
                    Long roomId = rtcUserService.getUserStayRoomId(i.getUserId());
                    if (roomId != null && roomId > 0) {
                        i.setIsParty(BoolIntVal.True);
                        i.setRoomId(roomId);
                        RtcRoom roomDO = rtcRoomService.getRoom(roomId);
                        if (roomDO != null) {
                            i.setRoomType(roomDO.getRoomType());
                        }

                    } else {
                        i.setIsParty(BoolIntVal.False);
                    }
                    UserDO tmpUser = userMap.get(i.getUserId());
                    if (tmpUser != null) {
                        i.setUserName(tmpUser.getUserName());
                        i.setUserAvatar(tmpUser.getUserAvatar());
                        Integer age = tmpUser.getUserAge();
                        if (age == null) {
                            age = 18;
                        }
                        i.setUserAge(age);
                        i.setUserSex(tmpUser.getUserSex());
                    }
                    if (adminIdSet.contains(i.getUserId())) {
                        i.setIsOfficial(BoolIntVal.True);
                    } else {
                        i.setIsOfficial(BoolIntVal.False);
                    }
                    i.setIsTop(0);
                });
                return feedList;
            }
        }
        return new ArrayList<>();
    }


    /**
     * 获取广场数据列表
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public PageList<FeedSquareVO> getSquareFeedList(Integer pageNum, Integer pageSize, Long userId) {
        PageList<FeedSquareVO> res = new PageList<>(pageNum, pageSize);
        Long now = System.currentTimeMillis();
        Long tenDaysAgo = now - 10 * DateUtil.DAY_OF_MILLION;
        List<Long> feedIds = getRandomFeedIds(pageSize);
        Long topFeed = 0L;
        if (pageNum != null && pageNum == 1) {
            topFeed = feedDAO.getTopFeed(now);
            if (topFeed != null && topFeed > 0) {
                feedIds.add(0, topFeed);
            }
        }
        List<FeedSquareVO> resList = getFeedSquareVOByFeedIds(feedIds, userId);
        if (topFeed != null && topFeed > 0) {
            if (CollectionUtil.isNotEmpty(resList)) {
                if (topFeed.equals(resList.get(0).getFeedId())) {
                    resList.get(0).setIsTop(1);
                }
                if (resList.size() > pageSize) {
                    resList = resList.subList(0, pageSize);
                }
            }
        }
        res.setList(resList);
        res.setTotal(feedDAO.getSquareFeedCount(tenDaysAgo, now));
        return res;
    }

    @Override
    public Map<Long, Integer> getThumbsNumMap(List<Long> feedIds) {
        if (CollectionUtil.isNotEmpty(feedIds)) {
            List<KeyValueMapDO> resList = feedThumbsupLogDAO.getThumbsNumList(feedIds);
            if (CollectionUtil.isNotEmpty(resList)) {
                Map<Long, Integer> res = new HashMap<>(resList.size());
                resList.forEach(i -> {
                    res.put(i.getMKey(), i.getMValue().intValue());
                });
                return res;
            }
        }
        return new HashMap<>();
    }

    private String handlerThumbsUpNum(Integer input) {
        if (input == null) {
            return "0";
        }
        String unit = "";
        if (input > 1000000) {
            unit = "M";
            return input / 1000000 + unit;
        }
        if (input > 1000) {
            unit = "K";
            return input / 1000 + unit;
        }
        return String.valueOf(input);
    }

    private String handlerStr(String input) {
        if (StringUtils.isBlank(input)) {
            return "0";
        }
        return input;
    }

    private Integer handlerBoolean(Boolean input) {
        if (input == null) {
            return BoolIntVal.False;
        }
        if (input) {
            return BoolIntVal.True;
        } else {
            return BoolIntVal.False;
        }
    }

    private Integer handleInteger(Integer input) {
        if (input == null) {
            return 0;
        } else {
            return input;
        }
    }


    private List<Long> getRandomFeedIds(Integer pageSize) {
        Long now = System.currentTimeMillis();
        // 保存1分钟数据
        Long fiveMinutesAfter = now + 60 * 1000;
        if (CollectionUtils.isEmpty(feedIdListBO.getFeedIds()) || feedIdListBO.getDateTime() < now) {
            Long tenDaysAgo = now - 10 * DateUtil.DAY_OF_MILLION;
            feedIdListBO.setFeedIds(feedDAO.getSquareFeedIdList(tenDaysAgo, now));
            feedIdListBO.setDateTime(fiveMinutesAfter);
        }
        int page = 20;
        if (pageSize != null) {
            page = pageSize;
        }
        int totalSize = feedIdListBO.getFeedIds().size();
        if (page > feedIdListBO.getFeedIds().size()) {
            page = totalSize;
        }
        List<Long> tmpList = new ArrayList<>(feedIdListBO.getFeedIds().size());
        tmpList.addAll(feedIdListBO.getFeedIds());
        Collections.shuffle(tmpList);
        return tmpList.subList(0, page);
    }


    @Override
    @Transactional
    public FeedThumbsUpResVO thumbsUp(Long feedId, Long userId) {
        Long now = System.currentTimeMillis();
        FeedThumbsupLogDO item = feedThumbsupLogDAO.getItemByParam(feedId, userId);
        Integer isUp = BoolIntVal.False;
        if (item != null) {
            feedThumbsupLogDAO.updateIsDel(item.getHisId(), BoolIntVal.True);
        } else {
            isUp = BoolIntVal.True;
            item = new FeedThumbsupLogDO();
            item.setHisId(identityService.getId());
            item.setUserId(userId);
            item.setFeedId(feedId);
            item.setCreateTime(now);
            item.setIsDel(BoolIntVal.False);
            feedThumbsupLogDAO.insert(item);

            // 发送点赞消息
            QueryWrapper qw = new QueryWrapper();
            qw.eq("userId", userId);
            qw.eq("feedId", feedId);
            qw.eq("isDel", BoolIntVal.True);
            if (feedThumbsupLogDAO.selectCount(qw) == 0) {
                QueueFeedLikeMsg msg = new QueueFeedLikeMsg();
                msg.setFeedId(feedId);
                msg.setUserId(feedDAO.selectUserId(feedId));
                msg.setOperatorUserId(userId);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FEED_LIKE, msg.serialize());
            }
        }
        FeedThumbsUpResVO res = new FeedThumbsUpResVO();
        res.setIsUp(isUp);
        Integer count = feedThumbsupLogDAO.getThumbsUpCount(feedId);
        res.setThumbsUpNum(String.valueOf(count));
        return res;
    }

    /**
     * 获取feed打赏数量
     */
    @Override
    public Map<Long, String> giftRewardMap(Collection<Long> feedIds) {
        return feedGiftRewardProvider.giftRewardMap(feedIds);
    }

    @Override
    public String newestTotalReward(long feedId, long rewardValue) {
        Map<Long, Long> statRewardMap = feedGiftRewardProvider.statRewardMap(ImmutableList.of(feedId));
        Long total = statRewardMap.get(feedId);
        total = total == null ? 0 : total;
        return feedGiftRewardProvider.getRewardStr(total + rewardValue);
    }

    /**
     * 设置用户信息
     *
     * @param list
     */
    @Override
    public <T> T setUserInfo(List<? extends UserTag> list) {
        if (CollectionUtils.isEmpty(list)) {
            return (T) list;
        }

        Map<Long, UserCardVO> userMap = getUserMap(list);
        for (UserTag tag : list) {
            if (tag == null) {
                continue;
            }
            Long userId = tag.getUserId();
            if (userId == null) {
                continue;
            }
            UserCardVO user = userMap.get(userId);
            if (user == null) {
                continue;
            }

            tag.setUserNo(user.getUserNo());
            tag.setUserName(user.getUserName());
            tag.setUserAvatar(user.getUserAvatar());
        }

        return (T) list;
    }

    /**
     * APP用户信息
     *
     * @param list UserTag 列表
     * @return key：用户Id value：用户信息
     */
    private Map<Long, UserCardVO> getUserMap(List<? extends UserTag> list) {
        Set<Long> userIds = getUserIds(list);
        return getUserMap(userIds);
    }

    /**
     * APP用户信息
     *
     * @param userIds APP用户Id列表
     * @return key：用户Id value：用户信息
     */
    private Map<Long, UserCardVO> getUserMap(Set<Long> userIds) {
        Map<Long, UserCardVO> userMap = new HashMap<>(userIds.size());
        if (userIds.isEmpty()) {
            return userMap;
        }
        List<UserCardVO> userList = userService.getNameAndAvatarByIds(new ArrayList<>(userIds));
        if (CollectionUtils.isEmpty(userList)) {
            return userMap;
        }

        for (UserCardVO user : userList) {
            if (user == null || user.getUserId() == null) {
                continue;
            }
            userMap.put(user.getUserId(), user);
        }

        return userMap;
    }

    private Set<Long> getUserIds(List<? extends UserTag> list) {
        Set<Long> userIds = new HashSet<>(list.size());
        for (UserTag tag : list) {
            if (tag == null || tag.getUserId() == null) {
                continue;
            }
            userIds.add(tag.getUserId());
        }

        return userIds;
    }

    /**
     * 随机获取一个三天内发表过状态的关注的用户
     *
     * @return
     */
    @Override
    public String getUserFollowName(Long userId) {
        Set<Long> userIds = socialRelationService.selectFollowsUserIds(userId);
        if (CollectionUtil.isNotEmpty(userIds)) {
            // 最近三天的数据
            Long threeDayBefore = System.currentTimeMillis() - DateUtil.DAY_OF_MILLION * 3;
            List<Long> feedUserIds = feedDAO.getFeedUserIds(userIds, threeDayBefore);
            if (CollectionUtil.isNotEmpty(feedUserIds)) {
                Long temp = feedUserIds.get(random.nextInt(feedUserIds.size()));
                UserDO userDO = userService.getUserById(temp);
                if (userDO != null) {
                    return userDO.getUserName() + " 发布了一条新动态";
                }
            }
        }
        return "";
    }
}
