package com.starbuds.server.service.feed.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.QueueFeedCreateMsg;
import com.starbuds.server.common.pojo.mq.QueueFeedViewMsg;
import com.starbuds.server.service.feed.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 普通队列消息监听
 *
 * @author Lay
 * @date 2020/1/3
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        log.info("service-feed收到rocketmq消息");
        try {
            switch (message.getTag()) {
                case RocketTag.FEED_CREATE:
                    // 内容创建消息
                    normalMsgHandler.handleFeedCreate(JSON.parseObject(message.getBody(), QueueFeedCreateMsg.class));
                case RocketTag.FEED_VIEW:
                    // 内容浏览消息
                    normalMsgHandler.handleFeedView(JSON.parseObject(message.getBody(), QueueFeedViewMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}
